#include "common.h"

#ifdef LIMAX_OS_UNIX_FAMILY
inline void closesocket(int s)
{
	::shutdown(s, SHUT_RDWR);
	::close(s);
}

inline int WSAGetLastError()
{
	return errno;
}
#endif

#ifdef LIMAX_OS_WINDOWS
typedef int32_t socklen_t;
#endif

namespace limax {
	TcpClient::~TcpClient() { }

	TcpClient::TcpClient(Listener* _listener, std::string ip, short port) :
		sock(::socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)), listener(_listener), closed(false), close_notified(false), r_task([this](){
		if (::connect(sock, (struct sockaddr*)&peer, sizeof(peer)))
		{
			_abort();
			return;
		}
		socklen_t len;
		::getsockname(sock, (struct sockaddr *)&local, &len);
		_connected();
		char buff[8192];
		while (true)
		{
			int32_t nrecv = (int32_t)::recv(sock, buff, sizeof(buff), 0);
			if (nrecv <= 0)
			{
				_close();
				return;
			}
			listener->onRecv(buff, nrecv);
		}
	}), w_task([this](){
		while (true)
		{
			Octets data = sq.get();
			int32_t l = (int32_t)data.size();
			if (l == 0)
				return;
			const char *p = (const char *)data.begin();
			while (true)
			{
				int32_t nsend = (int32_t)::send(sock, p, l, 0);
				if (nsend <= 0)
				{
					_close();
					return;
				}
				if (nsend == l)
					break;
				p += nsend;
				l -= nsend;
			}
		}
	})
	{
		memset(&peer, 0, sizeof(peer));
		peer.sin_family = AF_INET;
		peer.sin_addr.s_addr = inet_addr(ip.c_str());
		peer.sin_port = htons(port);
		int optval = 1;
		::setsockopt(sock, SOL_SOCKET, SO_KEEPALIVE, (const char *)&optval, sizeof(optval));
		::setsockopt(sock, IPPROTO_TCP, TCP_NODELAY, (const char *)&optval, sizeof(optval));
		listener->onCreate(this);
		Engine::execute(r_task);
		Engine::execute(w_task);
	}

	void TcpClient::create(Listener* _listener, std::string ip, short port)
	{
		new TcpClient(_listener, ip, port);
	}

	void TcpClient::close()
	{
		if (closed.exchange(true))
			return;
		::closesocket(sock);
		send(Octets());
	}

	void TcpClient::_close()
	{
		close();
		if (close_notified.exchange(true))
			return;
		listener->onClose(WSAGetLastError());
	}

	void TcpClient::_abort()
	{
		close();
		if (close_notified.exchange(true))
			return;
		listener->onAbort(peer);
	}

	void TcpClient::_connected()
	{
		if (closed.load())
			return;
		listener->onOpen(local, peer);
	}

	void TcpClient::send(Octets o)
	{
		sq.put(o);
	}

	void TcpClient::destroy()
	{
		close();
		r_task.join();
		w_task.join();
		delete this;
	}

} // namespace limax {

