﻿using limax.codec;
using limax.defines;
using limax.endpoint.auanyviews;
using limax.endpoint.providerendpoint;
using limax.endpoint.script;
using limax.endpoint.switcherendpoint;
using limax.endpoint.variant;
using limax.net;
using limax.providerendpoint;
using limax.util;
using System;
using System.Collections.Generic;
using System.Net;
using System.Numerics;
using System.Reflection;
using System.Threading;

namespace limax.endpoint
{
    public static class __ProtocolProcessManager
    {
        private static EndpointManagerImpl i(Protocol p)
        {
            return (EndpointManagerImpl)p.getManager();
        }
        public static void process(SHandShake p)
        {
            i(p).process(p);
        }
        public static void process(SessionKick p)
        {
            i(p).process(p);
        }
        public static void process(OnlineAnnounce p)
        {
            i(p).process(p);
        }
        public static void process(PingAndKeepAlive p)
        {
            i(p).process(p);
        }
        public static void process(SyncViewToClients p)
        {
            i(p).process(p);
        }
        public static void onResultViewOpen(ServiceResult view)
        {
            AuanyService.onResultViewOpen(view);
        }
    }
    internal abstract class AbstractViewContext : ViewContext
    {
        internal abstract void onSyncViewToClients(SyncViewToClients protocol);
        internal abstract void clear();
        public abstract ViewContextKind getKind();
        public abstract short getProviderId();
        public abstract View getSessionOrGlobalView(short classindex);
        public abstract TemporaryView findTemporaryView(short classindex, int instanceindex);
        public abstract EndpointManager getEndpointManager();
        public void sendMessage(View view, String msg)
        {
            SendControlToServer p = new SendControlToServer();
            p.providerid = getProviderId();
            p.classindex = view.getClassIndex();
            p.instanceindex = view is TemporaryView ? ((TemporaryView)view).getInstanceIndex() : 0;
            p.controlindex = 255;
            p.stringdata = msg;
            p.send(getEndpointManager().getTransport());
        }
    }
    public sealed class AuanyService
    {
        public delegate void ResultExperienceUpgrade(int errorSource, int errorCode);
        public delegate void ResultPay(int errorSource, int errorCode, String order);
        private static int snGenerator = 0;
        private readonly static IDictionary<int, AuanyService> map = new Dictionary<int, AuanyService>();
        private readonly int sn;
        private readonly object onresult;
        private readonly Timer timer;
        private static AuanyService removeService(int sn)
        {
            AuanyService service;
            lock (map)
            {
                if (map.TryGetValue(sn, out service))
                {
                    map.Remove(sn);
                    service.timer.Dispose();
                }
            }
            return service;
        }
        private AuanyService(object onresult, long timeout)
        {
            this.sn = Interlocked.Increment(ref snGenerator);
            this.onresult = onresult;
            timer = new Timer(_ =>
            {
                AuanyService service = removeService(sn);
                if (service != null)
                    service.onError(ErrorCodes.ENDPOINT_AUANY_SERVICE_CLIENT_TIMEOUT);
            });
            lock (map)
            {
                timer.Change(timeout, Timeout.Infinite);
                map.Add(this.sn, this);
            }
        }
        private void onError(int errorCode)
        {
            if (onresult is ResultExperienceUpgrade)
                onResultExperienceUpgrade(ErrorSource.ENDPOINT, errorCode);
            else if (onresult is ResultPay)
                onResultPay(ErrorSource.ENDPOINT, errorCode, null);
        }
        private void onResultExperienceUpgrade(int errorSource, int errorCode)
        {
            ((ResultExperienceUpgrade)onresult)(errorSource, errorCode);
        }
        private void onResultPay(int errorSource, int errorCode, String order)
        {
            ((ResultPay)onresult)(errorSource, errorCode, order);
        }
        internal static void onResultViewOpen(ServiceResult view)
        {
            view.registerListener("resultExperienceUpgrade", e =>
            {
                limax.auanyviews.ResultExperienceUpgrade r = (limax.auanyviews.ResultExperienceUpgrade)e.value;
                AuanyService service = removeService(r.sn);
                if (service != null)
                    service.onResultExperienceUpgrade(r.errorSource, r.errorCode);
            });
            view.registerListener("resultPay", e =>
            {
                limax.auanyviews.ResultPay r = (limax.auanyviews.ResultPay)e.value;
                AuanyService service = removeService(r.sn);
                if (service != null)
                    service.onResultPay(r.errorSource, r.errorCode, r.order);
            });
        }
        internal static void cleanup()
        {
            lock (map)
            {
                foreach (var service in map.Values)
                {
                    service.timer.Dispose();
                    service.onError(ErrorCodes.ENDPOINT_AUANY_SERVICE_ENGINE_CLOSE);
                }
                map.Clear();
            }
        }
        public static void experienceUpgrade(String username, String token, String platflag, long timeout,
        ResultExperienceUpgrade onresult, EndpointManager manager)
        {
            Service.getInstance(manager).ExperienceUpgrade(new AuanyService(onresult, timeout).sn, username, token,
                    platflag);
        }
        public static void experienceUpgrade(String username, String token, String platflag, long timeout,
                        ResultExperienceUpgrade onresult)
        {
            experienceUpgrade(username, token, platflag, timeout, onresult, Endpoint.getDefaultEndpointManager());
        }
        public static void pay(int gateway, int payid, int product, int price, int quantity, String receipt, long timeout, ResultPay onresult, EndpointManager manager)
        {
            Service.getInstance(manager).Pay(new AuanyService(onresult, timeout).sn, gateway, payid, product, price, quantity, receipt);
        }
        public static void pay(int gateway, int payid, int product, int price, int quantity, String receipt, long timeout, ResultPay onresult)
        {
            pay(gateway, payid, product, price, quantity, receipt, timeout, onresult, Endpoint.getDefaultEndpointManager());
        }
    }
    public static class Endpoint
    {
        private static ISet<short> protocolpvids = new HashSet<short>();
        public static void setEndpointState(State state)
        {
            limax.endpoint.states.Endpoint.EndpointClient.merge(state);
            foreach (var i in state.getSizePolicy())
                protocolpvids.Add((short)(i.Key >> 16));
        }
        public static void openEngine(int applicationthreadcount)
        {
            Engine.open(1, 1, applicationthreadcount);
        }
        public static void openEngine()
        {
            openEngine(4);
        }
        public static void closeEngine(Action done)
        {
            AuanyService.cleanup();
            new Thread(_ =>
            {
                try { Engine.close(); }
                finally
                {
                    if (done != null)
                        done();
                }
            }).Start();
        }
        public static EndpointConfigBuilder createLoginConfigBuilder(string serverIp, int serverPort, String username, String token, String platflag)
        {
            EndpointConfigBuilderImpl config = new EndpointConfigBuilderImpl(serverIp, serverPort, platflag, username, token, false);
            return config.isConfigCanLogin() ? config : null;
        }
        public static EndpointConfigBuilder createPingOnlyConfigBuilder(string serverIp, int serverPort)
        {
            return new EndpointConfigBuilderImpl(serverIp, serverPort, "", "", "", true);
        }
        private static void mapPvidsAppendValue(IDictionary<short, byte> pvids, short s, int nv)
        {
            byte v;
            if (pvids.TryGetValue(s, out v))
            {
                v = (byte)(v | nv);
                pvids.Remove(s);
            }
            else
                v = (byte)nv;
            pvids.Add(s, v);
        }
        private static ICollection<short> makeProtocolProviderIds(EndpointConfig config)
        {
            ISet<short> pvids = new HashSet<short>();
            foreach (int type in config.getEndpointState().getSizePolicy().Keys)
            {
                int pvid = type >> 16;
                if (pvid > 0)
                    pvids.Add((short)pvid);
            }
            return pvids;
        }
        private static ICollection<short> makeScriptProviderIds(EndpointConfig config)
        {
            if (config.getScriptEngineHandle() != null)
                return config.getScriptEngineHandle().getProviders();
            return new List<short>();
        }
        private static IDictionary<short, byte> makeProviderMap(EndpointConfig config)
        {
            var pvids = new Dictionary<short, byte>();
            foreach (short s in makeProtocolProviderIds(config))
                mapPvidsAppendValue(pvids, s, SessionType.ST_PROTOCOL);
            foreach (short s in config.getStaticViewClasses().Keys)
                mapPvidsAppendValue(pvids, s, SessionType.ST_STATIC);
            foreach (short s in config.getVariantProviderIds())
                mapPvidsAppendValue(pvids, s, SessionType.ST_VARIANT);
            foreach (short s in makeScriptProviderIds(config))
                mapPvidsAppendValue(pvids, s, SessionType.ST_SCRIPT);
            mapPvidsAppendValue(pvids, limax.endpoint.auanyviews.ViewManager.getInstance().getProviderId(), SessionType.ST_STATIC);
            return pvids;
        }
        public static void start(EndpointConfig config, EndpointListener listener)
        {
            new Thread(() =>
            {
                try
                {
                    if (null == Engine.getProtocolScheduler())
                        throw new Exception("endpoint need call openEngine");
                    IDictionary<short, byte> pvids = makeProviderMap(config);
                    if (!config.isPingServerOnly() && pvids.Count == 0)
                        throw new Exception("endpoint no available provider");
                    new EndpointManagerImpl(config, listener, pvids);
                }
                catch (Exception e)
                {
                    config.getClientManagerConfig().getDispatcher().execute(() => listener.onErrorOccured(ErrorSource.ENDPOINT, 0, e));
                }
            }).Start();
        }
        private static EndpointManagerImpl defaultEndpointManager;
        internal static void setDefaultEndpointManager(EndpointManagerImpl manager)
        {
            Interlocked.CompareExchange(ref defaultEndpointManager, manager, null);
        }
        internal static void clearDefaultEndpointManager(EndpointManagerImpl manager)
        {
            Interlocked.CompareExchange(ref defaultEndpointManager, null, manager);
        }
        public static EndpointManager getDefaultEndpointManager()
        {
            return defaultEndpointManager;
        }
    }
    public interface EndpointConfig : Config
    {
        int getDHGroup();
        ClientManagerConfig getClientManagerConfig();
        String getUserName();
        String getToken();
        String getPlatFlag();
        bool isPingServerOnly();
        State getEndpointState();
        IDictionary<short, IDictionary<short, Type>> getStaticViewClasses();
        ICollection<short> getVariantProviderIds();
        ScriptEngineHandle getScriptEngineHandle();
    }
    public interface EndpointConfigBuilder
    {
        EndpointConfigBuilder inputBufferSize(int inputBufferSize);
        EndpointConfigBuilder outputBufferSize(int outputBufferSize);
        EndpointConfigBuilder executor(Executor executor);
        EndpointConfigBuilder endpointState(params State[] states);
        EndpointConfigBuilder staticViewClasses(params View.StaticManager[] managers);
        EndpointConfigBuilder variantProviderIds(params short[] pvids);
        EndpointConfigBuilder scriptEngineHandle(ScriptEngineHandle handle);
        EndpointConfig build();
    }
    internal class EndpointConfigBuilderImpl : EndpointConfigBuilder
    {
        internal readonly string serverIp;
        internal readonly int serverPort;
        internal readonly string platflag;
        internal readonly string username;
        internal readonly string token;
        internal readonly bool pingServerOnly;
        internal volatile int _outputBufferSize = 8 * 1024;
        internal volatile int _inputBufferSize = 8 * 1024;
        internal volatile Executor _executor;
        internal volatile State _endpointState = new State();
        internal readonly IDictionary<short, IDictionary<short, Type>> _svclasses = new Dictionary<short, IDictionary<short, Type>>();
        internal ICollection<short> _variantpvids = new HashSet<short>();
        internal ScriptEngineHandle _scriptEngineHandle;
        private ClientManagerConfig clientConfig;
        public EndpointConfigBuilderImpl(string serverIp, int serverPort, String platflag,
                String username, String token, bool pingServerOnly)
        {
            this.serverIp = serverIp;
            this.serverPort = serverPort;
            this.platflag = platflag;
            this.username = username;
            this.token = token;
            this.pingServerOnly = pingServerOnly;
            this._executor = Engine.getApplicationExecutor().getExecutor(this);
            this.clientConfig = new DefaultClientManagerConfig(this);
            this._endpointState.merge(limax.endpoint.states.Endpoint.EndpointClient);
        }
        internal bool isConfigCanLogin()
        {
            if (username.Length <= 0)
                return false;
            if (token.Length <= 0)
                return false;
            if (platflag.Length <= 0)
                return false;
            return true;
        }
        public EndpointConfigBuilder inputBufferSize(int inputBufferSize)
        {
            this._inputBufferSize = inputBufferSize;
            return this;
        }
        public EndpointConfigBuilder outputBufferSize(int outputBufferSize)
        {
            this._outputBufferSize = outputBufferSize;
            return this;
        }
        public EndpointConfigBuilder endpointState(params State[] states)
        {
            _endpointState = new State();
            _endpointState.merge(limax.endpoint.states.Endpoint.EndpointClient);
            foreach (State state in states)
                _endpointState.merge(state);
            return this;
        }
        public EndpointConfigBuilder variantProviderIds(params short[] pvids)
        {
            _variantpvids.Clear();
            foreach (short id in pvids)
                _variantpvids.Add(id);
            return this;
        }
        public EndpointConfigBuilder staticViewClasses(params View.StaticManager[] managers)
        {
            _svclasses.Clear();
            foreach (View.StaticManager m in managers)
                _svclasses.Add(m.getProviderId(), m.getClasses());
            return this;
        }
        public EndpointConfigBuilder scriptEngineHandle(ScriptEngineHandle handle)
        {
            _scriptEngineHandle = handle;
            return this;
        }
        public EndpointConfigBuilder executor(Executor executor)
        {
            this._executor = executor;
            return this;
        }
        private class DefaultClientManagerConfig : ClientManagerConfig
        {
            private readonly EndpointConfigBuilderImpl impl;
            internal DefaultClientManagerConfig(EndpointConfigBuilderImpl impl)
            {
                this.impl = impl;
            }
            public String getName()
            {
                return "Endpoint";
            }
            public int getInputBufferSize()
            {
                return impl._inputBufferSize;
            }
            public int getOutputBufferSize()
            {
                return impl._outputBufferSize;
            }
            public bool isCheckOutputBuffer()
            {
                return true;
            }
            public byte[] getOutputSecurityBytes()
            {
                return null;
            }
            public byte[] getInputSecurityBytes()
            {
                return null;
            }
            public bool isOutputCompress()
            {
                return false;
            }
            public bool isInputCompress()
            {
                return false;
            }
            public State getDefaultState()
            {
                return limax.endpoint.states.Endpoint.getDefaultState();
            }
            public Dispatcher getDispatcher()
            {
                return new Dispatcher(impl._executor);
            }
            public IPEndPoint getPeerAddress()
            {
                return new IPEndPoint(IPAddress.Parse(impl.serverIp), impl.serverPort);
            }
            public bool isAutoReconnect()
            {
                return false;
            }
            public long getConnectTimeout()
            {
                return 5000;
            }
        };
        private class DefaultEndpointConfig : EndpointConfig
        {
            private readonly EndpointConfigBuilderImpl impl;
            internal DefaultEndpointConfig(EndpointConfigBuilderImpl impl)
            {
                this.impl = impl;
            }
            public int getDHGroup()
            {
                return 2;
            }
            public ClientManagerConfig getClientManagerConfig()
            {
                return impl.clientConfig;
            }
            public string getUserName()
            {
                return impl.username;
            }
            public string getToken()
            {
                return impl.token;
            }
            public string getPlatFlag()
            {
                return impl.platflag;
            }
            public bool isPingServerOnly()
            {
                return impl.pingServerOnly;
            }
            public Dispatcher getDispatcher()
            {
                return new Dispatcher(impl._executor);
            }
            public State getEndpointState()
            {
                return impl._endpointState;
            }
            public IDictionary<short, IDictionary<short, Type>> getStaticViewClasses()
            {
                return impl._svclasses;
            }
            public ICollection<short> getVariantProviderIds()
            {
                return impl._variantpvids;
            }
            public ScriptEngineHandle getScriptEngineHandle()
            {
                return impl._scriptEngineHandle;
            }
        }
        public EndpointConfig build()
        {
            return new DefaultEndpointConfig(this);
        }
    }
    public interface EndpointListener : ClientListener
    {
        void onSocketConnected();
        void onKeyExchangeDone();
        void onKeepAlived(int ms);
        void onErrorOccured(int source, int code, Exception exception);
    }
    public interface EndpointManager : ClientManager
    {
        long getSessionID();
        long getAccountFlags();
        ViewContext getViewContext(short pvid, ViewContextKind kind);
    }
    internal class EndpointManagerImpl : EndpointManager, ClientListener, SupportDispatch
    {
        private class ViewContextMap
        {
            private readonly IDictionary<short, ICollection<AbstractViewContext>> map = new Dictionary<short, ICollection<AbstractViewContext>>();
            internal void put(short pvid, AbstractViewContext vc)
            {
                ICollection<AbstractViewContext> c;
                if (!map.TryGetValue(pvid, out c))
                    map.Add(pvid, c = new List<AbstractViewContext>());
                c.Add(vc);
            }
            internal void onSyncViewToClients(SyncViewToClients protocol)
            {
                ICollection<AbstractViewContext> c;
                if (map.TryGetValue(protocol.providerid, out c))
                    foreach (AbstractViewContext vc in c)
                        vc.onSyncViewToClients(protocol);
            }
            internal void clear()
            {
                foreach (ICollection<AbstractViewContext> c in map.Values)
                    foreach (AbstractViewContext vc in c)
                        try { vc.clear(); }
                        catch (Exception) { }
                map.Clear();
            }
            internal ViewContext getViewContext(short pvid, ViewContextKind kind)
            {
                ICollection<AbstractViewContext> c;
                if (map.TryGetValue(pvid, out c))
                    foreach (AbstractViewContext vc in c)
                        if (kind == vc.getKind())
                            return vc;
                return null;
            }
        }
        internal ClientManager manager;
        private readonly EndpointConfig config;
        private readonly EndpointListener listener;
        private readonly IDictionary<short, byte> pvids;
        private readonly ScriptExchange scriptExchange;
        private readonly ViewContextMap viewContextMap = new ViewContextMap();
        enum LoginStatus
        {
            LOGINING, LOGINED_NOTIFY, LOGINED_DONE, RETRY,
        }
        private volatile LoginStatus loginstatus = LoginStatus.LOGINING;
        private readonly KeepAlive keepalive;
        private BigInteger dhRandom;
        private volatile Transport transportsaved = null;
        private volatile bool canretrylogin = true;
        private long sessionid = -1;
        private long accountflags = 0;
        public EndpointManagerImpl(EndpointConfig config, EndpointListener listener, IDictionary<short, byte> pvids)
        {
            this.config = config;
            this.listener = listener;
            this.pvids = pvids;
            this.manager = (ClientManager)Engine.add(config.getClientManagerConfig(), this, this);
            this.keepalive = new KeepAlive(this);
            this.scriptExchange = config.getScriptEngineHandle() != null ? new ScriptExchange(
                    this, config.getScriptEngineHandle()) : null;
            if (config.isPingServerOnly())
                canretrylogin = false;
        }
        public Transport getTransport()
        {
            return manager.getTransport();
        }
        public void close()
        {
            manager.close();
        }
        public void close(Transport transport)
        {
            manager.close(transport);
        }
        public Listener getListener()
        {
            return listener;
        }
        private class KeepAlive : IDisposable
        {
            private const int PING_TIMEOUT = 5;
            private const int KEEP_ALIVE_DELAY = 50;
            private readonly EndpointManagerImpl impl;
            private readonly Timer timer;
            private volatile bool wait_response;
            private volatile bool timeout = false;
            public KeepAlive(EndpointManagerImpl impl)
            {
                this.impl = impl;
                this.timer = new Timer(_ =>
                {
                    if (wait_response)
                    {
                        timeout = true;
                        Transport t = impl.getTransport();
                        if (t != null)
                            impl.close(t);
                    }
                    else
                    {
                        try
                        {
                            startPingAndKeepAlive(impl.getTransport());
                        }
                        catch (Exception e)
                        {
                            if (Trace.isErrorEnabled())
                                Trace.error("startPingAndKeepAlive", e);
                        }
                    }
                });
            }
            public void startPingAndKeepAlive(Transport transport)
            {
                new PingAndKeepAlive(DateTime.Now.Ticks / 10000).send(transport);
                wait_response = true;
                timer.Change(PING_TIMEOUT * 1000, Timeout.Infinite);
            }
            public void process(PingAndKeepAlive p)
            {
                if (timeout)
                    return;
                impl.listener.onKeepAlived((int)(DateTime.Now.Ticks / 10000 - p.timestamp));
                if (impl.config.isPingServerOnly())
                    return;
                wait_response = false;
                timer.Change(KEEP_ALIVE_DELAY * 1000, Timeout.Infinite);
            }
            public void Dispose() { timer.Dispose(); }
            public bool isTimeout() { return timeout; }
        }
        public void dispatch(Action runnable)
        {
            ((SupportDispatch)manager).dispatch(runnable);
        }
        public Manager getWrapperManager()
        {
            return null;
        }
        public void onManagerInitialized(Manager manager, Config config)
        {
            if (LoginStatus.RETRY == loginstatus)
                loginstatus = LoginStatus.LOGINING;
            else
                listener.onManagerInitialized(this, this.config);
        }
        public void onManagerUninitialized(Manager manager)
        {
            if (LoginStatus.RETRY == loginstatus)
            {

                Engine.getProtocolScheduler().execute(() =>
                {
                    try
                    {
                        manager = (ClientManager)Engine.add(config.getClientManagerConfig(), this, this);
                    }
                    catch (Exception e)
                    {
                        if (Trace.isErrorEnabled())
                            Trace.error("session logined retry", e);
                    }
                });
            }
            else
            {
                if (LoginStatus.LOGINED_DONE == loginstatus)
                {
                    try
                    {
                        keepalive.Dispose();
                        listener.onTransportRemoved(transportsaved);
                    }
                    catch (Exception e)
                    {
                        if (Trace.isErrorEnabled())
                            Trace.error("listener.onTransportRemoved", e);
                    }
                }
                if (LoginStatus.LOGINED_DONE == loginstatus || LoginStatus.LOGINED_NOTIFY == loginstatus)
                {
                    Endpoint.clearDefaultEndpointManager(this);
                    viewContextMap.clear();
                    if (scriptExchange != null)
                        scriptExchange.onUnload();
                }
                listener.onManagerUninitialized(this);
            }
        }
        public void onTransportAdded(Transport transport)
        {
            listener.onSocketConnected();
            if (config.isPingServerOnly())
            {
                keepalive.startPingAndKeepAlive(transport);
            }
            else
            {
                dhRandom = Helper.makeDHRandom();
                byte[] dh_data = Helper.generateDHResponse(config.getDHGroup(), dhRandom).ToByteArray();
                Array.Reverse(dh_data);
                new CHandShake((byte)config.getDHGroup(), Octets.wrap(dh_data)).send(transport);
            }
        }
        public void onTransportRemoved(Transport transport)
        {
            if (keepalive.isTimeout())
                onErrorOccured(ErrorSource.ENDPOINT, ErrorCodes.ENDPOINT_PING_TIMEOUT);
            if (LoginStatus.LOGINING == loginstatus)
            {
                if (!config.isPingServerOnly())
                    onAbort(transport);
            }
            else
                transportsaved = transport;
        }
        private void onErrorOccured(int source, int code)
        {
            listener.onErrorOccured(source, code, null);
        }
        public void onAbort(Transport transport)
        {
            listener.onAbort(transport);
        }
        public long getSessionID()
        {
            return sessionid;
        }
        public long getAccountFlags()
        {
            return accountflags;
        }
        public override string ToString()
        {
            return GetType().Name;
        }
        internal void process(SHandShake p)
        {
            StateTransport transport = (StateTransport)p.getTransport();
            byte[] dh_data = p.dh_data.getBytes();
            Array.Reverse(dh_data);
            byte[] material = Helper.computeDHKey(config.getDHGroup(), new BigInteger(dh_data), dhRandom).ToByteArray();
            Array.Reverse(material);
            byte[] key = transport.getPeerAddress().Address.GetAddressBytes();
            int half = material.Length / 2;
            HmacMD5 mac = new HmacMD5(Codec.Null(), key, 0, key.Length);
            mac.update(material, 0, half);
            transport.setOutputSecurityCodec(mac.digest(), p.c2sneedcompress);
            mac = new HmacMD5(Codec.Null(), key, 0, key.Length);
            mac.update(material, half, material.Length - half);
            transport.setInputSecurityCodec(mac.digest(), p.s2cneedcompress);
            transport.setState(limax.endpoint.states.Endpoint.EndpointSessionLogin);
            listener.onKeyExchangeDone();
            SessionLoginByToken send = new SessionLoginByToken();
            send.username = config.getUserName();
            send.token = config.getToken();
            send.platflag = config.getPlatFlag();
            foreach (var i in pvids)
                send.pvids.Add(i.Key, i.Value);
            key = transport.getLocalAddress().Address.GetAddressBytes();
            send.report_ip.replace(key);
            send.report_port = (short)transport.getLocalAddress().Port;
            send.send(transport);
            transport.setState(config.getEndpointState());
        }
        internal void process(SessionKick p)
        {
            onErrorOccured(ErrorSource.LIMAX, p.error);
            if (scriptExchange != null)
                scriptExchange.onClose(p.error);
        }
        internal void process(PingAndKeepAlive p)
        {
            keepalive.process(p);
        }
        internal void process(OnlineAnnounce p)
        {
            Transport transport = p.getTransport();
            if (ErrorSource.LIMAX == p.errorSource && ErrorCodes.SUCCEED == p.errorCode)
            {
                sessionid = p.sid;
                accountflags = p.flags;
                createStaticViewContextImpls();
                parseVariantDefines(p.variantdefines);
                if (scriptExchange != null)
                    scriptExchange.onLoad(p.scriptdefines);
                loginstatus = LoginStatus.LOGINED_NOTIFY;
                Endpoint.setDefaultEndpointManager(this);
                listener.onTransportAdded(transport);
                loginstatus = LoginStatus.LOGINED_DONE;
                keepalive.startPingAndKeepAlive(transport);
            }
            else
            {
                if (ErrorSource.LIMAX == p.errorSource && ErrorCodes.SWITCHER_SESSION_LOGINED == p.errorCode && canretrylogin)
                {
                    canretrylogin = false;
                    loginstatus = LoginStatus.RETRY;
                }
                else
                {
                    onErrorOccured(p.errorSource, p.errorCode);
                }
                close();
            }
        }
        private void parseVariantDefines(IDictionary<short, VariantDefines> variantdefines)
        {
            foreach (var e in variantdefines)
                viewContextMap.put(e.Key, VariantViewContextImpl.createInstance(e.Key, e.Value, this));
        }
        private void createStaticViewContextImpls()
        {
            foreach (var e in config.getStaticViewClasses())
                viewContextMap.put(e.Key, new StaticViewContextImpl(e.Key, e.Value, this));
            {
                var vm = limax.endpoint.auanyviews.ViewManager.getInstance();
                viewContextMap.put(vm.getProviderId(), new StaticViewContextImpl(vm.getProviderId(), vm.getClasses(), this));
            }
        }
        internal void process(SyncViewToClients p)
        {
            viewContextMap.onSyncViewToClients(p);
            if (scriptExchange != null)
                scriptExchange.onSyncViewToClients(p);
        }
        public ViewContext getViewContext(short pvid, ViewContextKind type)
        {
            return viewContextMap.getViewContext(pvid, type);
        }

    }
    internal class ScriptExchange : ScriptSender
    {
        private readonly EndpointManagerImpl netmanager;
        private readonly ScriptEngineHandle handle;
        private readonly ISet<short> providers;
        private object closeReason;
        public ScriptExchange(EndpointManagerImpl netmanager, ScriptEngineHandle handle)
        {
            this.netmanager = netmanager;
            this.handle = handle;
            this.providers = handle.getProviders();
        }
        private void process(int t, Object p)
        {
            if (handle.action(t, p) == 3)
                netmanager.close();
        }
        internal void onLoad(string welcome)
        {
            lock (this)
            {
                handle.registerScriptSender(this);
                process(1, welcome);
            }
        }
        internal void onSyncViewToClients(SyncViewToClients protocol)
        {
            lock (this)
            {
                if (protocol.stringdata.Length > 0
                        && providers.Contains(protocol.providerid))
                    process(1, protocol.stringdata);
            }
        }
        internal void onClose(object closeReason)
        {
            lock (this)
            {
                this.closeReason = closeReason;
            }
        }
        internal void onUnload()
        {
            try
            {
                lock (this)
                {
                    handle.action(2, closeReason);
                }
            }
            catch (Exception)
            {
            }
        }
        public Exception send(string s)
        {
            SendControlToServer protocol = new SendControlToServer();
            protocol.providerid = -1;
            protocol.stringdata = s;
            try
            {
                protocol.send(netmanager.getTransport());
            }
            catch (Exception e)
            {
                return e;
            }
            return null;
        }
    }
    internal class StaticViewContextImpl : AbstractViewContext
    {

        private readonly ViewContextImpl impl;
        private readonly IDictionary<short, Type> viewClasses;

        private class CreateViewInstance : ViewContextImpl.createViewInstance
        {
            private readonly short pvid;
            private readonly StaticViewContextImpl impl;
            public CreateViewInstance(short pvid, StaticViewContextImpl impl)
            {
                this.pvid = pvid;
                this.impl = impl;
            }
            public short getProviderId()
            {
                return pvid;
            }
            public View createView(short clsindex)
            {
                Type cls;
                if (!impl.viewClasses.TryGetValue(clsindex, out cls))
                    return null;
                ConstructorInfo constructor = cls.GetConstructor(BindingFlags.NonPublic | BindingFlags.Instance, null, new Type[] { typeof(ViewContext) }, null);
                return (View)constructor.Invoke(new object[] { impl });
            }
        }

        internal StaticViewContextImpl(short pvid, IDictionary<short, Type> map, EndpointManagerImpl netmanager)
        {
            viewClasses = new Dictionary<short, Type>(map);
            impl = new ViewContextImpl(new CreateViewInstance(pvid, this), netmanager);
        }
        public override View getSessionOrGlobalView(short classindex)
        {
            return impl.getSesseionOrGlobalView(classindex);
        }
        public override TemporaryView findTemporaryView(short classindex, int instanceindex)
        {
            return impl.findTemporaryView(classindex, instanceindex);
        }
        public override EndpointManager getEndpointManager()
        {
            return impl.getEndpointManager();
        }
        public override short getProviderId()
        {
            return impl.getProviderId();
        }
        public override ViewContextKind getKind()
        {
            return ViewContextKind.Static;
        }
        internal override void onSyncViewToClients(SyncViewToClients protocol)
        {
            impl.onSyncViewToClients(protocol);
        }
        internal override void clear()
        {
            impl.clear();
        }
    }
    public abstract class TemporaryView : View
    {
        internal volatile int instanceindex;
        protected TemporaryView(ViewContext vc) : base(vc) { }
        public int getInstanceIndex()
        {
            return instanceindex;
        }
        protected abstract void onOpen(ICollection<long> sessionids);
        protected abstract void onClose();
        protected abstract void onAttach(long sessionid);
        protected abstract void detach(long sessionid, byte reason);
        protected abstract void onDetach(long sessionid, byte reason);
        internal override void doClose()
        {
            base.doClose();
            onClose();
        }
        internal void doOnOpen(ICollection<long> sessionids)
        {
            onOpen(sessionids);
        }
        internal void doOnClose()
        {
            onClose();
        }
        internal void doOnAttach(long sessionid)
        {
            onAttach(sessionid);
        }
        internal void doDetach(long sessionid, byte reason)
        {
            detach(sessionid, reason);
        }

        public override string ToString()
        {
            return "[class = " + GetType().Name + " ProviderId = " + getViewContext().getProviderId() + " classindex = " + getClassIndex() + " instanceindex = " + instanceindex + "]";
        }
    }
    internal class VariantViewContextImpl : AbstractViewContext, SupportManageVariant
    {
        private class HollowTempViewHandler : TemporaryViewHandler
        {
            public void onOpen(VariantView view, ICollection<long> sessionids)
            {
            }
            public void onClose(VariantView view)
            {
            }
            public void onAttach(VariantView view, long sessionid)
            {
            }
            public void onDetach(VariantView view, long sessionid, int reason)
            {
            }
        }
        private readonly static TemporaryViewHandler hollowTempViewHandler = new HollowTempViewHandler();

        private readonly ViewContextImpl impl;
        private readonly IDictionary<short, object> viewdefines;
        private readonly ISet<string> tempviewnames;
        private readonly IDictionary<string, short> nametoidmap = new Dictionary<string, short>();
        private readonly IDictionary<string, TemporaryViewHandler> tempviewhandler = new Dictionary<string, TemporaryViewHandler>();
        private readonly VariantManager variantmanager;
        private readonly MethodInfo viewCreator;
        private class CreateViewInstance : ViewContextImpl.createViewInstance
        {
            private readonly short pvid;
            private readonly VariantViewContextImpl impl;
            internal CreateViewInstance(short pvid, VariantViewContextImpl impl)
            {
                this.pvid = pvid;
                this.impl = impl;
            }
            public short getProviderId()
            {
                return pvid;
            }
            public View createView(short classindex)
            {
                return impl.createViewInstance(classindex, impl);
            }

        }

        private VariantViewContextImpl(short pvid, IDictionary<short, object> viewdefines, ISet<string> tempviewnames, EndpointManagerImpl netmanager)
        {
            this.impl = new ViewContextImpl(new CreateViewInstance(pvid, this), netmanager);
            this.viewdefines = viewdefines;
            this.tempviewnames = tempviewnames;
            foreach (var e in viewdefines)
                nametoidmap.Add(e.Value.ToString(), e.Key);
            viewCreator = typeof(VariantManager).GetMethod("createDynamicViewInstance", BindingFlags.NonPublic | BindingFlags.Static, null, new Type[] { typeof(short), typeof(object), typeof(TemporaryViewHandler), typeof(ViewContext) }, null);
            ConstructorInfo constructor = typeof(VariantManager).GetConstructor(BindingFlags.NonPublic | BindingFlags.Instance, null, new Type[] { typeof(SupportManageVariant), typeof(ViewContext) }, null);
            variantmanager = (VariantManager)constructor.Invoke(new object[] { this, this });
        }

        internal View createViewInstance(short index, ViewContext vc)
        {
            object vd;
            if (!viewdefines.TryGetValue(index, out vd))
                return null;
            return (View)viewCreator.Invoke(null, new object[] { impl.getProviderId(), vd, getTemporaryViewHandler(vd.ToString(), true), vc });
        }
        public short getViewClassIndex(string name)
        {
            short id;
            if (nametoidmap.TryGetValue(name, out id)) return id;
            throw new Exception("getViewClassIndex id not found[] name " + name);
        }
        public void setTemporaryViewHandler(string name, TemporaryViewHandler handler)
        {
            lock (this)
            {
                if (!tempviewnames.Contains(name))
                    throw new Exception("temporary view named \"" + name + "\" not exist");
                tempviewhandler.Add(name, handler);
            }
        }
        public TemporaryViewHandler getTemporaryViewHandler(string name, bool returnDeafault)
        {
            lock (this)
            {
                TemporaryViewHandler handler;
                if (tempviewhandler.TryGetValue(name, out handler))
                    return handler;
                return returnDeafault ? hollowTempViewHandler : null;
            }
        }
        public VariantManager getVariantManager()
        {
            return variantmanager;
        }
        public override ViewContextKind getKind()
        {
            return ViewContextKind.Variant;
        }
        public override View getSessionOrGlobalView(short classindex)
        {
            return impl.getSesseionOrGlobalView(classindex);
        }
        public override TemporaryView findTemporaryView(short classindex, int instanceindex)
        {
            return impl.findTemporaryView(classindex, instanceindex);
        }
        public override EndpointManager getEndpointManager()
        {
            return impl.getEndpointManager();
        }
        public override short getProviderId()
        {
            return impl.getProviderId();
        }
        internal override void onSyncViewToClients(SyncViewToClients protocol)
        {
            impl.onSyncViewToClients(protocol);
        }
        internal override void clear()
        {
            impl.clear();
        }
        internal static VariantViewContextImpl createInstance(short pvid, VariantDefines defines, EndpointManagerImpl netmanager)
        {
            IDictionary<short, object> viewdefines = new Dictionary<short, object>();
            ISet<string> tempviewnames = new HashSet<string>();
            MethodInfo method = typeof(VariantManager).GetMethod("parseViewDefines", BindingFlags.NonPublic | BindingFlags.Static, null, new Type[] { typeof(VariantDefines), typeof(IDictionary<short, object>), typeof(ISet<string>) }, null);
            method.Invoke(null, new object[] { defines, viewdefines, tempviewnames });
            return new VariantViewContextImpl(pvid, viewdefines, tempviewnames, netmanager);
        }
    }
    public abstract class View
    {
        private readonly AbstractViewContext viewContext;
        protected View(ViewContext vc)
        {
            viewContext = (AbstractViewContext)vc;
        }
        public ViewContext getViewContext()
        {
            return viewContext;
        }
        public void sendMessage(string msg)
        {
            viewContext.sendMessage(this, msg);
        }
        public abstract short getClassIndex();
        protected abstract void onData(long sessionid, byte index, byte field, Octets data, Octets dataremoved);
        internal void doOnData(long sessionid, byte index, byte field, Octets data, Octets dataremoved)
        {
            onData(sessionid, index, field, data, dataremoved);
        }
        public override string ToString()
        {
            return "[class = " + GetType().Name + " ProviderId = " + viewContext.getProviderId() + " classindex = " + getClassIndex() + "]";
        }
        abstract protected ISet<string> getFieldNames();
        private readonly IDictionary<string, ViewChangedListener> listeners = new Dictionary<string, ViewChangedListener>();
        public Action registerListener(ViewChangedListener listener)
        {
            lock (this)
            {
                foreach (string name in getFieldNames())
                {
                    if (listeners.ContainsKey(name))
                        listeners[name] += listener;
                    else
                        listeners[name] = listener;
                }
                return () =>
                {
                    lock (this)
                    {
                        foreach (string name in getFieldNames())
                            listeners[name] -= listener;
                    }
                };
            }
        }
        public Action registerListener(string fieldname, ViewChangedListener listener)
        {
            lock (this)
            {
                if (!getFieldNames().Contains(fieldname))
                    throw new Exception("View \"" + GetType().Name + "\" has no field named \"" + fieldname + "\"");
                if (listeners.ContainsKey(fieldname))
                    listeners[fieldname] += listener;
                else
                    listeners[fieldname] = listener;
                return () =>
                {
                    lock (this)
                    {
                        listeners[fieldname] -= listener;
                    }
                };
            }
        }
        protected void onViewChanged(long sessionid, String fieldname, object value, ViewChangedType type)
        {
            ViewChangedListener l;
            if (listeners.TryGetValue(fieldname, out l))
                l(new ViewChangedEvent(this, sessionid, fieldname, value, type));
        }
        internal virtual void doClose()
        {
            lock (this)
            {
                listeners.Clear();
            }
        }
        public abstract class Control : Marshal
        {
            private readonly View view;
            protected Control(View view)
            {
                this.view = view;
            }
            public abstract short getViewClassIndex();
            private int getViewInstanceIndex()
            {
                var tv = view as TemporaryView;
                return tv == null ? 0 : tv.getInstanceIndex();
            }
            public abstract byte getControlIndex();
            public void send()
            {
                SendControlToServer protocol = new SendControlToServer();
                protocol.providerid = view.getViewContext().getProviderId();
                protocol.classindex = this.getViewClassIndex();
                protocol.instanceindex = this.getViewInstanceIndex();
                protocol.controlindex = this.getControlIndex();
                protocol.controlparameter = new OctetsStream().marshal(this);
                protocol.send(view.getViewContext().getEndpointManager().getTransport());
            }
            public abstract OctetsStream marshal(OctetsStream os);
            public abstract OctetsStream unmarshal(OctetsStream os);
        }
        public interface StaticManager
        {
            short getProviderId();
            IDictionary<short, Type> getClasses();
        }
    }
    public struct ViewChangedEvent
    {
        readonly View _view;
        readonly long _sessionid;
        readonly String _fieldname;
        readonly object _value;
        readonly ViewChangedType _type;
        internal ViewChangedEvent(View view, long sessionid, String fieldname, object value, ViewChangedType type)
        {
            _view = view;
            _sessionid = sessionid;
            _fieldname = fieldname;
            _value = value;
            _type = type;
        }
        public View view { get { return _view; } }
        public long sessionid { get { return _sessionid; } }
        public String fieldname { get { return _fieldname; } }
        public object value { get { return _value; } }
        public ViewChangedType type { get { return _type; } }
        public override string ToString()
        {
            return _view + " " + _sessionid + " " + _fieldname + " " + _value + " " + type;
        }
    }
    public delegate void ViewChangedListener(ViewChangedEvent e);
    public enum ViewChangedType
    {
        NEW, REPLACE, TOUCH, DELETE
    }
    public enum ViewContextKind
    {
        Static, Variant, Script
    }
    public interface ViewContext
    {
        View getSessionOrGlobalView(short classindex);
        TemporaryView findTemporaryView(short classindex, int instanceindex);
        EndpointManager getEndpointManager();
        short getProviderId();
        ViewContextKind getKind();
        void sendMessage(View view, String msg);
    }
    internal class ViewContextImpl
    {
        internal interface createViewInstance
        {
            short getProviderId();
            View createView(short classindex);
        }
        private static long getTemporaryViewKey(short classindex, int instanceindex)
        {
            return ((classindex & 0xFFFFL) << 32) | (instanceindex & 0xFFFFFFFFL);
        }
        internal readonly EndpointManagerImpl netmanager;
        private readonly createViewInstance createview;
        private readonly IDictionary<short, View> viewmap = new Dictionary<short, View>();
        private readonly IDictionary<long, TemporaryView> tempviewmap = new Dictionary<long, TemporaryView>();
        internal ViewContextImpl(createViewInstance createview,
                EndpointManagerImpl netmanager)
        {
            this.netmanager = netmanager;
            this.createview = createview;
        }
        internal short getProviderId()
        {
            return createview.getProviderId();
        }
        internal void clear()
        {
            lock (this)
            {
                foreach (var v in tempviewmap.Values)
                    v.doClose();
                foreach (var v in viewmap.Values)
                    v.doClose();
                viewmap.Clear();
                tempviewmap.Clear();
            }
        }
        internal TemporaryView findTemporaryView(short classindex, int instanceindex)
        {
            lock (this)
            {
                TemporaryView view;
                return tempviewmap.TryGetValue(getTemporaryViewKey(classindex, instanceindex), out view) ? view : null;
            }
        }
        internal void onSyncViewToClients(SyncViewToClients p)
        {
            switch (p.synctype)
            {
                case SyncViewToClients.DT_VIEW_DATA:
                    {
                        View view = getViewInstance(p.classindex);
                        lock (view)
                        {
                            foreach (ViewVariableData var in p.vardatas)
                                view.doOnData(netmanager.getSessionID(), var.index, var.field, var.data, var.dataremoved);
                        }
                        break;
                    }
                case SyncViewToClients.DT_TEMPORARY_INIT_DATA:
                    {
                        TemporaryView view = getTemporaryView(p.classindex, p.instanceindex);
                        lock (view)
                        {
                            ICollection<long> sessionids = new HashSet<long>();
                            foreach (ViewMemberData item in p.members)
                                sessionids.Add(item.sessionid);
                            view.doOnOpen(sessionids);
                            foreach (ViewVariableData var in p.vardatas)
                                view.doOnData(netmanager.getSessionID(), var.index, var.field, var.data, var.dataremoved);
                            foreach (ViewMemberData var in p.members)
                                view.doOnData(var.sessionid, var.vardata.index, var.vardata.field, var.vardata.data, var.vardata.dataremoved);
                        }
                        break;
                    }
                case SyncViewToClients.DT_TEMPORARY_DATA:
                    {
                        TemporaryView view = findTemporaryView(p.classindex, p.instanceindex);
                        if (view != null)
                        {
                            lock (view)
                            {
                                foreach (ViewVariableData var in p.vardatas)
                                    view.doOnData(netmanager.getSessionID(), var.index, var.field, var.data, var.dataremoved);
                                foreach (ViewMemberData var in p.members)
                                    view.doOnData(var.sessionid, var.vardata.index, var.vardata.field, var.vardata.data, var.vardata.dataremoved);
                            }
                        }
                        else
                        {
                            Trace.error("unopened temporary view  pvid = " + p.providerid + " classindex = " + p.classindex + " instanceindex = " + p.instanceindex);
                        }
                        break;
                    }
                case SyncViewToClients.DT_TEMPORARY_ATTACH:
                    {
                        TemporaryView view = findTemporaryView(p.classindex, p.instanceindex);
                        if (view != null)
                        {
                            lock (view)
                            {
                                view.doOnAttach(p.members[0].sessionid);
                                foreach (ViewMemberData var in p.members)
                                    view.doOnData(var.sessionid, var.vardata.index, var.vardata.field, var.vardata.data, var.vardata.dataremoved);
                            }
                        }
                        else
                        {
                            Trace.error("unopened temporary view pvid = " + p.providerid + " classindex = " + p.classindex + " instanceindex = " + p.instanceindex + " or error members.size() = " + p.members.Count);
                        }
                        break;
                    }
                case SyncViewToClients.DT_TEMPORARY_DETACH:
                    {
                        TemporaryView view = findTemporaryView(p.classindex, p.instanceindex);
                        if (view != null && p.members.Count == 1)
                        {
                            ViewMemberData e = p.members[0];
                            lock (view)
                            {
                                view.doDetach(e.sessionid, e.vardata.index);
                            }
                        }
                        else
                        {
                            Trace.error("unopened temporary view  pvid = " + p.providerid + " classindex = " + p.classindex + " instanceindex = " + p.instanceindex + " or error members.size() = " + p.members.Count);
                        }
                        break;
                    }
                case SyncViewToClients.DT_TEMPORARY_CLOSE:
                    {
                        TemporaryView view = closeTemporaryView(p.classindex,
                               p.instanceindex);
                        if (null != view)
                            lock (view)
                            {
                                view.doClose();
                            }
                        break;
                    }
            }
        }
        private View getViewInstance(short classindex)
        {
            lock (this)
            {
                View view;
                if (viewmap.TryGetValue(classindex, out view))
                    return view;
                view = createview.createView(classindex);
                if (null == view)
                    throw new Exception("unknown view class pvid = " + createview.getProviderId() + " classindex = " + classindex);
                return view;
            }
        }
        private TemporaryView getTemporaryView(short classindex, int instanceindex)
        {
            lock (this)
            {
                long key = getTemporaryViewKey(classindex, instanceindex);
                TemporaryView view;
                if (!tempviewmap.TryGetValue(key, out view))
                {
                    view = (TemporaryView)createview.createView(classindex);
                    if (null == view)
                        throw new Exception("unknown temp view class pvid = " + createview.getProviderId() + " classindex = " + classindex + " instanceindex = " + instanceindex);
                    view.instanceindex = instanceindex;
                    tempviewmap.Add(key, view);
                }
                return view;
            }
        }
        private TemporaryView closeTemporaryView(short classindex, int instanceindex)
        {
            lock (this)
            {
                long key = getTemporaryViewKey(classindex, instanceindex);
                TemporaryView view;
                if (tempviewmap.TryGetValue(key, out view))
                {
                    tempviewmap.Remove(key);
                    return view;
                }
                return null;
            }
        }
        internal View getSesseionOrGlobalView(short classindex)
        {
            lock (this)
            {
                View view;
                if (viewmap.TryGetValue(classindex, out view))
                    return view;
                view = createview.createView(classindex);
                if (null != view)
                    viewmap.Add(view.getClassIndex(), view);
                return view;
            }
        }
        internal EndpointManager getEndpointManager()
        {
            return netmanager;
        }
    }
    public delegate void ViewVisitor<T>(T value);
}
