package limax.auany;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import limax.auanymonitor.AuthPlat;
import limax.auanymonitor.AuthProvider;
import limax.defines.SessionFlags;
import limax.zdb.Procedure;

final class GetSessionProcedure implements Procedure {

	private final String username;
	private final String uid;
	private final String plat;
	private final Collection<Short> pvids;
	private long sessionid = -1;
	private long flags = 0;
	private Set<Integer> permission = Collections.emptySet();

	public GetSessionProcedure(String plat, boolean isExperience, String uid, String username,
			Collection<Short> pvids) {
		this.username = username;
		this.uid = uid;
		this.plat = plat;
		this.pvids = pvids;
		this.flags = isExperience ? SessionFlags.FLAG_EXPERIENCE_ACCOUNT : 0;
	}

	public long getSessionId() {
		return sessionid;
	}

	public long getFlags() {
		return flags;
	}

	public Set<Integer> getPermission() {
		return permission;
	}

	@Override
	public boolean process() throws Exception {
		final cbean.SessionKey key = new cbean.SessionKey(plat, uid);
		xbean.SessionInfo info = table.Sessioninfos.update(key);
		if (null == info) {
			sessionid = table.Sessions.insert(key).getKey();
			info = table.Sessioninfos.insert(key);
			info.setSessionid(sessionid);
			info.setUsername(username);
			info.setFlags(flags);
			permission = Collections.emptySet();

			AuthPlat.increment_newaccount(plat);
			pvids.forEach(pvid -> AuthProvider.increment_newaccount(plat, pvid));
		} else {
			sessionid = info.getSessionid();
			flags = info.getFlags();
			if (SessionFlags.FLAG_HAS_PERMISSION == (flags & SessionFlags.FLAG_HAS_PERMISSION)) {
				xbean.SessionPermission sp = table.Sessionpermissions.select(sessionid);
				if (null == sp)
					permission = Collections.emptySet();
				else
					permission = new HashSet<>(sp.getPermission());
			}

			AuthPlat.increment_auth(plat);
			pvids.forEach(pvid -> AuthProvider.increment_auth(plat, pvid));
		}
		return true;
	}

}
