package limax.auany.auanyviews;

import limax.auany.__ProtocolProcessManager;
import limax.defines.ErrorCodes;
import limax.defines.ErrorSource;
import limax.endpoint.AuanyService.ResultExperienceUpgrade;
import limax.endpoint.AuanyService.ResultPay;
import limax.provider.ViewContext;
import limax.provider.ViewStub;
import limax.util.Resource;
import limax.util.Trace;

public final class Service extends limax.auany.auanyviews._Service {

	private Service(ViewContext context, ViewStub stub, Resource resource) {
		super(context, stub, resource);
	}

	@Override
	protected void onClose() {
	}

	@Override
	protected void onControl(_Service.ExperienceUpgrade param, long sessionid) {
		String info = "";
		if (Trace.isInfoEnabled())
			info = "experienceupgrade!" + param.username + "," + param.token + "," + param.platflag;
		__ProtocolProcessManager.onExperienceUpgrade(cbResultExperienceUpgrade(sessionid, param.sn, info), sessionid,
				param.username, param.token, param.platflag);
	}

	@Override
	protected void onControl(_Service.Pay param, long sessionid) {
		String info = "";
		if (Trace.isInfoEnabled())
			info = "payorder!" + param.gateway + "," + param.payid + "," + param.product + "," + param.price + ","
					+ param.quantity + "," + param.receipt;
		__ProtocolProcessManager.onPay(sessionid, param.gateway, param.payid, param.product, param.price,
				param.quantity, param.receipt, cbResultPay(sessionid, param.sn, info));
	}

	@Override
	protected void onMessage(String message, long sessionid) {
		int index = message.indexOf('=');
		if (-1 == index)
			return;
		String cmd = message.substring(0, index);
		final String[] args = message.substring(index + 1).split(",");
		int sn;
		try {
			sn = Integer.parseInt(args[0]);
		} catch (NumberFormatException e) {
			return;
		}
		switch (cmd) {
		case "experienceupgrade": {
			ResultExperienceUpgrade onresult = cbResultExperienceUpgrade(sessionid, sn, message);
			if (args.length != 4) {
				onresult.accept(ErrorSource.LIMAX, ErrorCodes.AUANY_SERVICE_BAD_ARGS);
				return;
			}
			__ProtocolProcessManager.onExperienceUpgrade(onresult, sessionid, args[1], args[2], args[3]);
			return;
		}
		case "pay": {
			ResultPay onresult = cbResultPay(sessionid, sn, message);
			if (args.length != 7) {
				onresult.accept(ErrorSource.LIMAX, ErrorCodes.AUANY_SERVICE_BAD_ARGS, "");
				return;
			}
			int gateway, payid, product, price, quantity;
			try {
				gateway = Integer.parseInt(args[1]);
				payid = Integer.parseInt(args[2]);
				product = Integer.parseInt(args[3]);
				price = Integer.parseInt(args[4]);
				quantity = Integer.parseInt(args[5]);
			} catch (NumberFormatException e) {
				onresult.accept(ErrorSource.LIMAX, ErrorCodes.AUANY_SERVICE_BAD_ARGS, "");
				return;
			}
			if (price <= 0 || quantity <= 0) {
				onresult.accept(ErrorSource.LIMAX, ErrorCodes.AUANY_SERVICE_BAD_ARGS, "");
				return;
			}
			__ProtocolProcessManager.onPay(sessionid, gateway, payid, product, price, quantity, args[6], onresult);
			return;
		}
		}
	}

	private static ResultExperienceUpgrade cbResultExperienceUpgrade(long sessionid, int sn, String info) {
		return (errorSource, errorCode) -> {
			ServiceResult view = ServiceResult.createInstance();
			view.info = info;
			view._setResultExperienceUpgrade(new limax.auanyviews.ResultExperienceUpgrade(sn, errorSource, errorCode));
			view.getMemberShip().add(sessionid);
		};
	}

	private static ResultPay cbResultPay(long sessionid, int sn, String info) {
		return (errorSource, errorCode, order) -> {
			ServiceResult view = ServiceResult.createInstance();
			view.info = info;
			view._setResultPay(new limax.auanyviews.ResultPay(sn, errorSource, errorCode, order));
			view.getMemberShip().add(sessionid);
		};
	}
}
