package limax.endpoint;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

import limax.defines.ErrorCodes;
import limax.defines.ErrorSource;
import limax.endpoint.auanyviews.Service;
import limax.endpoint.auanyviews.ServiceResult;
import limax.net.Engine;

public final class AuanyService {
	public interface ResultExperienceUpgrade {
		void accept(int errorSource, int errorCode);
	}

	public interface ResultPay {
		void accept(int errorSource, int errorCode, String order);
	}

	private final static AtomicInteger snGenerator = new AtomicInteger();
	private final static Map<Integer, AuanyService> map = new HashMap<Integer, AuanyService>();

	private final int sn;
	private final Object onresult;
	private final Future<?> future;

	private static AuanyService removeService(int sn) {
		synchronized (map) {
			AuanyService service = map.remove(sn);
			if (service != null)
				service.future.cancel(false);
			return service;
		}
	}

	private AuanyService(final Object onresult, long timeout) {
		this.sn = snGenerator.getAndIncrement();
		this.onresult = onresult;
		synchronized (map) {
			this.future = Engine.getProtocolScheduler().schedule(new Runnable() {
				@Override
				public void run() {
					AuanyService service = removeService(sn);
					if (service != null)
						service.onError(ErrorCodes.ENDPOINT_AUANY_SERVICE_CLIENT_TIMEOUT);
				}
			}, timeout, TimeUnit.MILLISECONDS);
			map.put(this.sn, this);
		}
	}

	private void onError(int errorCode) {
		if (onresult instanceof ResultExperienceUpgrade)
			onResultExperienceUpgrade(ErrorSource.ENDPOINT, errorCode);
		else if (onresult instanceof ResultPay)
			onResultPay(ErrorSource.ENDPOINT, errorCode, null);
	}

	private void onResultExperienceUpgrade(int errorSource, int errorCode) {
		((ResultExperienceUpgrade) onresult).accept(errorSource, errorCode);
	}

	private void onResultPay(int errorSource, int errorCode, String order) {
		((ResultPay) onresult).accept(errorSource, errorCode, order);
	}

	static void onResultViewOpen(ServiceResult view) {
		view.registerListener("resultExperienceUpgrade", new ViewChangedListener() {
			@Override
			public void onViewChanged(ViewChangedEvent e) {
				limax.auanyviews.ResultExperienceUpgrade r = (limax.auanyviews.ResultExperienceUpgrade) e.getValue();
				AuanyService service = removeService(r.sn);
				if (service != null)
					service.onResultExperienceUpgrade(r.errorSource, r.errorCode);
			}
		});
		view.registerListener("resultPay", new ViewChangedListener() {
			@Override
			public void onViewChanged(ViewChangedEvent e) {
				limax.auanyviews.ResultPay r = (limax.auanyviews.ResultPay) e.getValue();
				AuanyService service = removeService(r.sn);
				if (service != null)
					service.onResultPay(r.errorSource, r.errorCode, r.order);
			}
		});
	}

	static void cleanup() {
		synchronized (map) {
			for (AuanyService service : map.values()) {
				service.future.cancel(false);
				service.onError(ErrorCodes.ENDPOINT_AUANY_SERVICE_ENGINE_CLOSE);
			}
			map.clear();
		}
	}

	public static void experienceUpgrade(String username, String token, String platflag, long timeout,
			ResultExperienceUpgrade onresult, EndpointManager manager) throws Exception {
		Service.getInstance(manager).ExperienceUpgrade(new AuanyService(onresult, timeout).sn, username, token,
				platflag);
	}

	public static void experienceUpgrade(String username, String token, String platflag, long timeout,
			ResultExperienceUpgrade onresult) throws Exception {
		experienceUpgrade(username, token, platflag, timeout, onresult, Endpoint.getDefaultEndpointManager());
	}

	public static void pay(int gateway, int payid, int product, int price, int quantity, String receipt, long timeout,
			ResultPay onresult, EndpointManager manager) throws Exception {
		Service.getInstance(manager).Pay(new AuanyService(onresult, timeout).sn, gateway, payid, product, price,
				quantity, receipt);
	}

	public static void pay(int gateway, int payid, int product, int price, int quantity, String receipt, long timeout,
			ResultPay onresult) throws Exception {
		pay(gateway, payid, product, price, quantity, receipt, timeout, onresult, Endpoint.getDefaultEndpointManager());
	}
}
