package limax.xmlgen.csharp;

import java.io.File;
import java.io.PrintStream;
import java.util.Collection;

import limax.util.StringUtils;
import limax.xmlgen.FileOperation;
import limax.xmlgen.Main;
import limax.xmlgen.Manager;
import limax.xmlgen.Protocol;
import limax.xmlgen.Service;
import limax.xmlgen.State;

final class StatesFormatter {
	private final Manager manager;
	private final String namespace;

	StatesFormatter(Service service, Manager manager) {
		this.manager = manager;
		this.namespace = service.getFullName();
	}

	void makeStates() {
		final File output = new File(Main.outputPath, Xmlgen.path_xmlgen);

		try (final PrintStream ps = FileOperation.fopen(output, manager.getName().toLowerCase() + ".states.cs")) {
			BeanFormatter.printCommonInclude(ps);
			ps.println();
			final Collection<State> sortstates = manager.getStates();
			ps.println("namespace " + namespace + ".states");
			ps.println("{");
			ps.println("	static public class " + manager.getName());
			ps.println("	{");
			ps.println();
			for (final State state : sortstates) {
				final String outstatename = StringUtils.upper1(state.getName());

				ps.println("		static public limax.net.State create" + outstatename + "() {");
				ps.println("			limax.net.State state = new limax.net.State();");
				for (final Protocol p : state.getProtocols())
					ps.println("			state.addStub(typeof(" + namespace + "." + p.getFullName() + "), "
							+ p.getType() + ", " + p.getMaxsize() + ");");
				ps.println("			return state;");
				ps.println("		}");
				ps.println();
			}
			for (final State state : sortstates) {
				final String outstatename = StringUtils.upper1(state.getName());
				ps.println(
						"		static public limax.net.State " + state.getName() + " = create" + outstatename + "();");
			}
			ps.println();
			ps.println("		static public limax.net.State getDefaultState() {");
			ps.println("			return " + manager.getInitStateName() + ";");
			ps.println("		}");

			ps.println("	}");
			ps.println("}");
			ps.println();

		}
	}

}
