package limax.xmlgen.java;

import java.io.PrintStream;

import limax.xmlgen.Bind;
import limax.xmlgen.Variable;

class BindFormatter {
	private final Bind bind;
	private final boolean genServer;

	public BindFormatter(Bind bind, boolean genServer) {
		this.bind = bind;
		this.genServer = genServer;
	}

	public void make(PrintStream ps) {
		if (bind.isFullBind())
			return; // full bean import need not generate
		ps.println("	" + (genServer ? "private" : "public") + " static class " + bind.getName()
				+ " implements limax.codec.Marshal" + (bind.attachment() == null ? "" : ", limax.codec.StringMarshal")
				+ " {");
		printDefine(ps);
		ps.println("	}");
		ps.println();
	}

	public void printDefine(PrintStream ps) {
		// declare variables
		for (Variable v : bind.getVariables()) {
			ps.println("		public " + TypeName.getName(v.getType()) + " " + v.getName() + "; " + v.getComment());
		}
		ps.println("");

		if (genServer)
			Copy.make(bind, ps, "		");
		else
			Construct.make(bind, ps, "		");
		Marshal.make(bind.getVariables(), ps, "		");
		if (bind.attachment() != null)
			StringMarshal.make(bind.getVariables(), ps, "		");
		Unmarshal.make(bind, ps, "		");
		if (!genServer)
			Equals.make(bind, ps, "		");
	}
}
